"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ern_core_1 = require("ern-core");
const ern_api_gen_1 = require("ern-api-gen");
const lodash_1 = __importDefault(require("lodash"));
const chalk_1 = __importDefault(require("chalk"));
const path_1 = __importDefault(require("path"));
const ApiImplAndroidGenerator_1 = __importDefault(require("./android/ApiImplAndroidGenerator"));
const ApiImplIosGenerator_1 = __importDefault(require("./ios/ApiImplIosGenerator"));
const ApiImplJsGenerator_1 = __importDefault(require("./js/ApiImplJsGenerator"));
const NullApiImplGenerator_1 = __importDefault(require("./NullApiImplGenerator"));
class ApiImplGen {
    async generateApiImplementation(apiPackagePath, paths, reactNativeVersion, platforms, regen = false) {
        ern_core_1.log.debug(`Inside generateApiImplementation for api:${apiPackagePath.toString()},  platforms:${platforms.toString()}`);
        const schemaJson = path_1.default.join(paths.outDirectory, 'node_modules', apiPackagePath.name, 'schema.json');
        const apis = await ern_api_gen_1.ApiGenUtils.extractApiEventsAndRequests(schemaJson);
        this.updatePackageJsonWithApiNames(paths.outDirectory, apis);
        const reactNativeElectrodeBridge = await (0, ern_core_1.readPackageJson)(path_1.default.join(paths.outDirectory, 'node_modules/react-native-electrode-bridge'));
        const generators = this.getGenerators(platforms);
        for (const generator of generators) {
            try {
                if (generator) {
                    await generator.generate(apiPackagePath, paths, reactNativeVersion, [
                        apiPackagePath,
                        ern_core_1.PackagePath.fromString(`react-native@${reactNativeVersion}`),
                        ern_core_1.PackagePath.fromString(`react-native-electrode-bridge@${reactNativeElectrodeBridge.version}`),
                    ], apis, regen);
                }
            }
            catch (e) {
                throw new Error(`API implementation project generation failed: ${e}`);
            }
        }
        ern_core_1.log.info(chalk_1.default.green(`API implementation project was successfully generated in ${paths.outDirectory}`));
    }
    async getDependencies(apiPackagePath) {
        try {
            ern_core_1.log.info('Looking for peerDependencies');
            const { dependencies, peerDependencies } = await ern_core_1.yarn.info(apiPackagePath);
            const pluginsNames = [];
            if (peerDependencies) {
                this.pushDependencyNames(peerDependencies, pluginsNames);
            }
            if (dependencies) {
                this.pushDependencyNames(dependencies, pluginsNames);
            }
            if (pluginsNames.length === 0) {
                ern_core_1.log.info(`no other dependencies found for ${apiPackagePath.name}`);
            }
            return lodash_1.default.map(pluginsNames, ern_core_1.PackagePath.fromString);
        }
        catch (e) {
            throw new Error(`getDependencies: ${e}`);
        }
    }
    pushDependencyNames(dependencies, pluginsNames) {
        for (const dependency of Object.keys(dependencies)) {
            pluginsNames.push(`${dependency}@${dependencies[dependency]}`);
        }
        return pluginsNames;
    }
    getGenerators(platforms) {
        return lodash_1.default.map(platforms, (platform) => {
            switch (platform) {
                case 'android':
                    return new ApiImplAndroidGenerator_1.default();
                case 'ios':
                    return new ApiImplIosGenerator_1.default();
                case 'js':
                    return new ApiImplJsGenerator_1.default();
                default:
                    return new NullApiImplGenerator_1.default();
            }
        });
    }
    updatePackageJsonWithApiNames(outputDirectoryPath, apis) {
        const packageJson = (0, ern_core_1.readPackageJsonSync)(outputDirectoryPath);
        packageJson.ern.containerGen.apiNames = lodash_1.default.map(apis, (api) => api.apiName);
        (0, ern_core_1.writePackageJsonSync)(outputDirectoryPath, packageJson);
    }
}
exports.default = ApiImplGen;
//# sourceMappingURL=ApiImplGen.js.map