"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ern_core_1 = require("ern-core");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
class ApiImplJsGenerator {
    get name() {
        return 'ApiImplJsGenerator';
    }
    get platform() {
        return 'js';
    }
    async generate(apiDependency, paths, reactNativeVersion, plugins, apis, regen) {
        ern_core_1.log.debug(`Starting project generation for ${this.platform}`);
        this.regenerateApiImpl = regen;
        await this.fillHull(apiDependency, paths, apis);
    }
    async fillHull(apiDependency, paths, apis) {
        ern_core_1.shell.pushd(ern_core_1.shell.pwd());
        try {
            const outputDirectory = path_1.default.join(paths.outDirectory, 'js');
            ern_core_1.log.debug(`Creating out directory(${outputDirectory}) for JS.`);
            await fs_extra_1.default.ensureDir(outputDirectory);
            const mustacheFile = path_1.default.join(ern_core_1.Platform.currentPlatformVersionPath, 'ern-api-impl-gen/resources/js/apiimpl.mustache');
            for (const api of apis) {
                api.packageName = apiDependency.name;
                await ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(mustacheFile, api, path_1.default.join(outputDirectory, `${api.apiName}ApiImpl.js`));
            }
            const indexMustacheFile = path_1.default.join(ern_core_1.Platform.currentPlatformVersionPath, 'ern-api-impl-gen/resources/js/index.mustache');
            const apisMustacheView = {
                apis,
            };
            await ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(indexMustacheFile, apisMustacheView, path_1.default.join(paths.outDirectory, 'index.js'));
        }
        finally {
            ern_core_1.shell.popd();
        }
    }
}
exports.default = ApiImplJsGenerator;
//# sourceMappingURL=ApiImplJsGenerator.js.map